"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = __importStar(require("vscode"));
const libPath = __importStar(require("path"));
const libFs = __importStar(require("fs"));
const crypto = __importStar(require("crypto"));
const utils = __importStar(require("./utils"));
const consts = __importStar(require("./constant"));
const config_provider_1 = require("./config-provider");
class ConfigExplorer {
    constructor(root, context, agent) {
        this.context = context;
        this.agent = agent;
        this.spaceState = this.context.workspaceState;
        this.updateCommit = {};
        this.loading = {};
        this.itemsCount = 0;
        this.invalid = [];
        this.notLatest = [];
        this.notCompleted = [];
        this.completed = [];
        this.description = '';
        this.limitUpTaskHandle = () => { };
        this.provider = new config_provider_1.ConfigTreeProvider(root, context.workspaceState);
        this.treeViewer = vscode.window.createTreeView('hiConfigTree', {
            treeDataProvider: this.provider
        });
        this.treeViewer.onDidChangeVisibility(() => {
            this.provider.refresh();
        });
        this.limitUpTaskHandle = utils.genLimitTask(this, 10);
        context.subscriptions.push(vscode.commands.registerCommand('configExplorer.viewConfigLink', (node) => {
            this.showLink(node);
        }), vscode.commands.registerCommand('configExplorer.viewDetail', (node) => {
            this.view(node);
        }), vscode.commands.registerCommand('configExplorer.openConfig', (resource) => {
            this.open(resource);
        }), vscode.commands.registerCommand('configExplorer.compare', (node) => {
            this.compare(node);
        }), vscode.commands.registerCommand('configExplorer.comparePrev', (node) => {
            this.comparePrev(node);
        }), vscode.commands.registerCommand('configExplorer.addTag', (node) => {
            this.changeTag(node);
        }), vscode.commands.registerCommand('configExplorer.changeTag', (node) => {
            this.changeTag(node);
        }), vscode.commands.registerCommand('configExplorer.updateConfig', (node) => {
            this.updateAll(node);
        }), vscode.commands.registerCommand('configExplorer.downloadConfig', (node) => {
            this.downloadAll(node);
        }), vscode.commands.registerCommand('configExplorer.downloadConfigForce', (node) => {
            this.downloadAll(node, true);
        }), vscode.commands.registerCommand('configExplorer.refresh', (node) => {
            this.provider.refresh();
        }), vscode.commands.registerCommand('configExplorer.sync', (node) => {
            this.provider.initTree(consts.downloadType.update);
        }), vscode.commands.registerCommand('configExplorer.syncForce', (node) => {
            this.provider.initTree(consts.downloadType.force);
        }), vscode.commands.registerCommand('configExplorer.clearMemento', (node) => {
            this.provider.clearState();
            clearInterval(this.uploadInterval);
        }));
    }
    showLink(item) {
        utils.getConfigList(item.detail).then((result) => {
            let list = result
                .sort((a, b) => a.config_id - b.config_id)
                .map((config) => {
                let url = utils.getProp(config, consts.fileUrlPath);
                return {
                    id: config.config_id,
                    url
                };
            });
            vscode.commands.executeCommand('extension.showLink');
            this.agent.panel.webview.postMessage({
                cmd: 'renderList',
                data: { name: item.label, list }
            });
        });
    }
    view(item) {
        vscode.commands.executeCommand('extension.showDetail');
        this.agent.panel.webview.postMessage({
            cmd: 'renderDetail',
            data: item.detail
        });
    }
    saveConfigFile(item, parentPath, { commitCode, isOpen, isForce, isSpec }) {
        return __awaiter(this, void 0, void 0, function* () {
            let post = Object.assign(Object.assign(Object.assign({}, utils.genParam(item)), item.detail), { commitCode });
            let name = utils.getConfigName(item.detail);
            if (isSpec) {
                name = item.detail.config_id + '(' + name + ')';
            }
            let fullPath = libPath.join(utils.getRoot(vscode), parentPath, name);
            let empty = isForce || !libFs.existsSync(fullPath);
            if (empty) {
                post.force = true;
                yield utils.createConfigFile(post, parentPath, name);
            }
            isOpen && vscode.window.showTextDocument(vscode.Uri.file(fullPath));
            return fullPath;
        });
    }
    open(item) {
        this.saveConfigFile(item, item.path, { isOpen: true });
        let spaceState = this.spaceState;
        let statePath = 'config_collect' + utils.genCommonConfigPath(item.path);
        if (!spaceState.get(statePath)) {
            spaceState.update(statePath, JSON.stringify(utils.getCollectDetail(item)));
        }
    }
    compare(item) {
        return __awaiter(this, void 0, void 0, function* () {
            let post = utils.genParam(item);
            post.config_id = item.detail.config_id;
            let latest = yield utils.getLatestConfig(post);
            let clone = utils.deepClone(item.detail);
            let currentPath = yield this.saveConfigFile(item, item.path, {
                isOpen: false
            });
            item.detail = latest;
            let prevPath = yield this.saveConfigFile(item, consts.diffFolder, {
                isOpen: false,
                isForce: true,
                isSpec: true
            });
            item.detail = clone;
            vscode.commands.executeCommand('vscode.diff', vscode.Uri.file(currentPath), vscode.Uri.file(prevPath));
        });
    }
    comparePrev(item) {
        return __awaiter(this, void 0, void 0, function* () {
            let currentCommit = item.detail.commit_code;
            if (currentCommit <= 1) {
                utils.msg(consts.msgText.noOldVersion, consts.msgType.warn);
                return;
            }
            let currentPath = yield this.saveConfigFile(item, item.path, {
                isOpen: false
            });
            let prevPath = yield this.saveConfigFile(item, consts.diffFolder, {
                commitCode: currentCommit - 1,
                isOpen: false,
                isForce: true,
                isSpec: true
            });
            vscode.commands.executeCommand('vscode.diff', vscode.Uri.file(currentPath), vscode.Uri.file(prevPath));
        });
    }
    changeTag(item) {
        let post = utils.genParam(item);
        post.config_id = item.detail.config_id;
        this.checkLatest({
            item,
            post
        }, ({ result }) => {
            if (result.tag !== item.detail.tag) {
                utils.msg(consts.msgText.notLatestTag, consts.msgType.warn);
                return;
            }
            let options = {
                prompt: 'tag_name:',
                placeHolder: consts.hintText.tag
            };
            vscode.window
                .showInputBox(options)
                .then((value) => {
                if (utils.checkValue(value)) {
                    return Promise.reject();
                }
                post.param = {
                    tag: (value === null || value === void 0 ? void 0 : value.toLowerCase()) || ''
                };
                let loadKey = 'load_' + post.collect + post.config_id;
                if (this.loading[loadKey] === true) {
                    return;
                }
                this.loading[loadKey] = true;
                utils
                    .updateConfig(post)
                    .then(() => {
                    utils.msg(consts.msgText.tagChangeOk);
                    this.provider.refresh();
                })
                    .catch((err) => {
                    utils.msg(consts.msgText.ajaxFail, consts.msgType.error);
                    utils.createLogFile({
                        type: 'changeTag',
                        message: err
                    });
                })
                    .finally(() => {
                    this.loading[loadKey] = false;
                });
            })
                .then(undefined, (err) => { });
        });
    }
    updateAll(node) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!node.detail.hasOwnProperty('collect')) {
                let elem = node;
                if (this.preCheck(elem)) {
                    this.callUpdate(elem);
                }
            }
            else {
                utils.msg(consts.msgText.upChecking);
                let itemList = yield utils.getConfigList(node.detail);
                let child = this.provider.genTreeCollectChildren(itemList, node);
                let uploadList = [];
                // precheck modified configs
                child.forEach((item) => {
                    let tagChild = utils.getProp(item, 'detail.children');
                    if (!tagChild) {
                        this.preCheck(item, uploadList);
                    }
                    else if (tagChild.length) {
                        let tagItemList = this.provider.genTreeItemList(tagChild, item);
                        tagItemList.forEach((item) => {
                            this.preCheck(item, uploadList);
                        });
                    }
                });
                let uploadTotal = uploadList.length;
                if (!uploadTotal) {
                    utils.msg(consts.msgText.noModifyAll, consts.msgType.warn);
                    return;
                }
                // upload all modified
                this.editDescription((value) => {
                    if (value) {
                        this.description = value;
                    }
                    utils.msg(consts.msgText.fileUploadAll(uploadList));
                    this.itemsCount = uploadTotal;
                    this.uploadInterval = setInterval(() => {
                        utils.msg('已处理 ' + (uploadTotal - this.itemsCount) + '/' + uploadTotal);
                    }, 5000);
                    uploadList.forEach((item) => {
                        this.callUpdate(item, true);
                    });
                });
            }
        });
    }
    callUpdate(item, multi) {
        let fields = ['config_id', 'config_file', 'description', 'tag'];
        let post = utils.genParam(item);
        post.config_id = item.detail.config_id;
        post.filepath = item.fullPath;
        post.param = utils.pick(item.detail, fields);
        // check valid
        let buffer = libFs.readFileSync(item.fullPath);
        let str = buffer.toString();
        let valid = false;
        try {
            JSON.parse(str);
            valid = true;
        }
        catch (err) {
            console.log('JSON parse error!');
        }
        if (!valid) {
            if (multi) {
                this.checkResultAll(item, {
                    type: 'invalid'
                });
            }
            else {
                utils.msg(consts.msgText.notJson, consts.msgType.warn);
            }
            return;
        }
        // check commit code
        this.limitUpTaskHandle({
            item,
            post,
            multi
        }, this.checkLatest, ({ item, post, spaceState, configItemObj, configItemPath }) => {
            if (multi) {
                post.param.description = this.description || '';
                this.upload({
                    item,
                    post,
                    spaceState,
                    configItemObj,
                    configItemPath,
                    multi
                });
            }
            else {
                this.editDescription((value) => {
                    post.param.description = value || '';
                    this.upload({
                        item,
                        post,
                        spaceState,
                        configItemObj,
                        configItemPath
                    });
                });
            }
        });
    }
    upload({ item, post, spaceState, configItemObj, configItemPath, multi }) {
        let loadKey = 'load_' + post.collect + post.config_id;
        if (this.loading[loadKey] === true) {
            return;
        }
        this.loading[loadKey] = true;
        if (!multi) {
            utils.msg(consts.msgText.uploading);
        }
        return utils
            .uploadConfig(post)
            .then((result) => {
            this.updateCommit[item.detail.config_id] = result.commit_code;
            let publishUrl = utils.getCollectDetail(item).publish;
            return utils.prePublish(publishUrl, result, post);
        })
            .then(() => {
            if (multi) {
                this.checkResultAll(item, {
                    type: 'completed'
                });
            }
            else {
                utils.msg(consts.msgText.fileUploadOk);
            }
        })
            .catch((err) => {
            if (multi) {
                this.checkResultAll(item, {
                    type: 'notCompleted'
                });
            }
            else if (err) {
                let errMsg = consts.msgText.fileUploadFail;
                if (err.type === 'notfound') {
                    errMsg = err.msg;
                }
                else if (err.interface) {
                    errMsg = err.msg.msg;
                }
                utils.msg(errMsg, consts.msgType.error);
            }
            utils.createLogFile({
                type: 'editConfig',
                message: err
            });
        })
            .finally(() => {
            if (!multi || this.itemsCount <= 0) {
                this.provider.refresh();
                let state = Object.assign(Object.assign({}, configItemObj), this.updateCommit);
                spaceState.update(configItemPath, JSON.stringify(state));
                this.updateCommit = {};
            }
            this.loading[loadKey] = false;
        });
    }
    preCheck(item, modified) {
        // check hash
        let fsHash = crypto.createHash('sha256');
        let buffer;
        try {
            buffer = libFs.readFileSync(item.fullPath);
        }
        catch (err) {
            return false;
        }
        fsHash.update(buffer);
        let hash = fsHash.digest('hex');
        if (hash === item.sha256) {
            if (!modified) {
                utils.msg(consts.msgText.noModify, consts.msgType.warn);
            }
            return false;
        }
        else if (modified) {
            modified.push(item);
        }
        return true;
    }
    editDescription(callback) {
        let options = {
            prompt: 'description:',
            placeHolder: consts.hintText.desc
        };
        vscode.window
            .showInputBox(options)
            .then((value) => {
            if (utils.checkValue(value)) {
                return Promise.reject();
            }
            callback(value);
        })
            .then(undefined, (err) => { });
    }
    downloadAll(node, isForce) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!node.detail.hasOwnProperty('collect')) {
                utils.msg(consts.msgText.downloading);
                let item = node;
                this.download(item, true)
                    .then(() => {
                    this.provider.refresh();
                    utils.msg(consts.msgText.downloadOk + '：' + item.configName);
                })
                    .catch(() => {
                    utils.msg(consts.msgText.downloadFail, consts.msgType.error);
                });
            }
            else {
                utils.msg(consts.msgText.checking);
                // 配置列表
                let itemList = yield utils.getConfigList(node.detail);
                let downloadList = itemList;
                // 非强制下载模式时，需要获取所有需要更新的条目，用于后期判断是否全部处理完毕
                if (!isForce) {
                    // 检测时，使用配置列表形式
                    let configItemPath = 'config_item' + utils.genCommonConfigPath(node.path);
                    let configItemObj = utils.readConfigState(this.spaceState, configItemPath);
                    downloadList = yield utils.getDownloadList(itemList, {
                        detail: utils.deepClone(node.detail),
                        configItemObj,
                        path: node.path
                    });
                }
                if (!downloadList.length) {
                    utils.msg(consts.msgText.downloadStop, consts.msgType.warn);
                    this.provider.refresh();
                    return;
                }
                // 节点列表
                let child = this.provider.genTreeCollectChildren(downloadList, node);
                let flag = {
                    count: 0,
                    pass: [],
                    stop: []
                };
                let limit = 20;
                let limitDownTaskHandle = utils.genLimitTask(this, limit);
                utils.msg(consts.msgText.downloadAll(downloadList));
                let downloadInterval;
                if (downloadList.length > limit) {
                    downloadInterval = setInterval(() => {
                        utils.msg('已下载 ' + flag.pass.length + '/' + downloadList.length);
                    }, 5000);
                }
                // 下载时，使用节点列表形式
                child.forEach((item) => {
                    let tagChild = utils.getProp(item, 'detail.children');
                    if (!tagChild) {
                        limitDownTaskHandle({
                            item,
                            isForce,
                            flag,
                            length: downloadList.length
                        }, this.callDownload, () => { }, () => {
                            clearInterval(downloadInterval);
                        });
                    }
                    else if (tagChild.length) {
                        let tagItemList = this.provider.genTreeItemList(tagChild, item);
                        tagItemList.forEach((item) => {
                            limitDownTaskHandle({
                                item,
                                isForce,
                                flag,
                                length: downloadList.length
                            }, this.callDownload, () => { }, () => {
                                clearInterval(downloadInterval);
                            });
                        });
                    }
                });
            }
        });
    }
    callDownload({ item, isForce, flag, length }, callback, lastCallback) {
        return this.download(item, isForce)
            .then(() => {
            flag.pass.push(item.detail.config_id);
            callback && callback();
        })
            .catch(() => {
            flag.stop.push(item.detail.config_id);
        })
            .finally(() => {
            flag.count++;
            if (flag.count >= length) {
                if (flag.pass.length === length) {
                    let msg = consts.msgText.downloadAllOk({ length });
                    utils.msg(msg);
                }
                else {
                    let msg = consts.msgText.downloadFail + '：' + flag.stop.join('/');
                    utils.msg(msg, consts.msgType.error);
                }
                this.provider.refresh();
                lastCallback && lastCallback();
            }
        });
    }
    download(item, isForce) {
        let post = utils.genParam(item);
        post.config_id = item.detail.config_id;
        let spaceState = this.spaceState;
        let configItemPath = utils.genItemConfigPath(item, post);
        let configItemObj = utils.readConfigState(spaceState, configItemPath);
        return utils.getLatestConfig(post).then((result) => __awaiter(this, void 0, void 0, function* () {
            let name = utils.getConfigName(result);
            let fileDetail = Object.assign(Object.assign({}, utils.getCollectDetail(item)), result);
            let localCommit = configItemObj[result.config_id];
            if (!localCommit || localCommit < result.commit_code || isForce) {
                fileDetail.force = true;
            }
            yield utils.createConfigFile(fileDetail, item.path, name);
            configItemObj = utils.readConfigState(spaceState, configItemPath);
            configItemObj[result.config_id] = result.commit_code;
            spaceState.update(configItemPath, JSON.stringify(configItemObj));
        }));
    }
    checkLatest({ item, post, multi }, callback) {
        let spaceState = this.spaceState;
        let configItemPath = utils.genItemConfigPath(item, post);
        let configItemObj = utils.readConfigState(spaceState, configItemPath);
        return utils.getLatestConfig(post).then((result) => {
            let localCommit = configItemObj[item.detail.config_id];
            if (!localCommit || localCommit < result.commit_code) {
                if (multi) {
                    this.checkResultAll(item, {
                        type: 'notLatest'
                    });
                }
                else {
                    utils.msg(consts.msgText.notLatestFile, consts.msgType.warn);
                }
            }
            else {
                callback({ item, post, result, spaceState, configItemObj, configItemPath });
            }
        });
    }
    checkResultAll(item, { type }, callback) {
        this.itemsCount--;
        this[type].push({
            configId: item.detail.config_id,
            fileUrl: utils.getProp(item.detail, consts.fileUrlPath)
        });
        if (this.itemsCount === 0) {
            clearInterval(this.uploadInterval);
            let record = this.invalid.concat(this.notLatest, this.notCompleted);
            if (record.length) {
                let completed = this.genTips(this.completed, '上传成功的配置：');
                let invalid = this.genTips(this.invalid, '', 'JSON 格式错误');
                let notLatest = this.genTips(this.notLatest, '', '本地未下载最新版');
                let notCompleted = this.genTips(this.notCompleted, '', '发布失败');
                utils.msg(`${completed} 上传失败的配置：${invalid}${notLatest}${notCompleted}`, consts.msgType.warn);
            }
            else {
                if (callback) {
                    callback();
                }
                else {
                    utils.msg(consts.msgText.fileUploadAllOk(this.completed));
                }
            }
            this.invalid.length = 0;
            this.notLatest.length = 0;
            this.notCompleted.length = 0;
            this.completed.length = 0;
            this.description = '';
        }
    }
    genTips(arr, label = '', suffix) {
        let ids = utils.flatFromArr(arr, 'configId');
        let reason = suffix ? `(${suffix})` : '';
        return arr.length ? `${label}${ids.length}个${reason};` : '';
    }
}
exports.default = ConfigExplorer;
//# sourceMappingURL=config-explorer.js.map