"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeItem = exports.TreeTag = exports.TreeCollect = exports.TreeApp = exports.TreeEnv = void 0;
const vscode = __importStar(require("vscode"));
const libPath = __importStar(require("path"));
const libFs = __importStar(require("fs"));
const utils = __importStar(require("./utils"));
class TreeEnv extends vscode.TreeItem {
    constructor(label, collapsibleState, detail, spaceState) {
        super(label, collapsibleState);
        this.label = label;
        this.collapsibleState = collapsibleState;
        this.detail = detail;
        this.spaceState = spaceState;
        this.iconPath = {
            light: utils.lightRes('server.svg'),
            dark: utils.darkRes('server.svg')
        };
        // this.initAuth();
    }
    get path() {
        return this.label;
    }
    get authkey() {
        return this._authkey;
    }
    initAuth() {
        if (this.spaceState) {
            this._authkey = utils.readAuth(this.spaceState, this.label);
            if (!this._authkey) {
                utils
                    .getAuth(this.detail)
                    .then((result) => {
                    this._authkey = result;
                    utils.writeAuth(this.spaceState, this.label, result);
                })
                    .catch((err) => {
                    utils.createLogFile({
                        type: 'getAuth',
                        message: err
                    });
                });
            }
        }
    }
}
exports.TreeEnv = TreeEnv;
class TreeApp extends vscode.TreeItem {
    constructor(label, collapsibleState, parent, detail) {
        super(label, collapsibleState);
        this.label = label;
        this.collapsibleState = collapsibleState;
        this.parent = parent;
        this.detail = detail;
        this.iconPath = {
            light: utils.lightRes('application.svg'),
            dark: utils.darkRes('application.svg')
        };
        this.contextValue = 'appItem';
    }
    get path() {
        return libPath.join(this.parent.path, this.label);
    }
    get fullPath() {
        return libPath.join(utils.getRoot(vscode), this.path);
    }
}
exports.TreeApp = TreeApp;
class TreeCollect extends vscode.TreeItem {
    constructor(label, collapsibleState, parent, detail) {
        super(label, collapsibleState);
        this.label = label;
        this.collapsibleState = collapsibleState;
        this.parent = parent;
        this.detail = detail;
        this.iconPath = {
            light: utils.lightRes('collection.svg'),
            dark: utils.darkRes('collection.svg')
        };
        this.contextValue = 'collectItem';
    }
    get path() {
        return libPath.join(this.parent.path, this.label);
    }
    get fullPath() {
        return libPath.join(utils.getRoot(vscode), this.path);
    }
}
exports.TreeCollect = TreeCollect;
class TreeTag extends vscode.TreeItem {
    constructor(label, collapsibleState, parent, detail) {
        super(label, collapsibleState);
        this.label = label;
        this.collapsibleState = collapsibleState;
        this.parent = parent;
        this.detail = detail;
        this.iconPath = {
            light: utils.lightRes('tag.svg'),
            dark: utils.darkRes('tag.svg')
        };
        this.contextValue = 'tagItem';
    }
    get path() {
        return this.parent.path;
    }
}
exports.TreeTag = TreeTag;
class TreeItem extends vscode.TreeItem {
    constructor(label, collapsibleState, parent, detail) {
        super(label, collapsibleState);
        this.label = label;
        this.collapsibleState = collapsibleState;
        this.parent = parent;
        this.detail = detail;
        this.isExist = libFs.existsSync(this.fullPath);
        // command: 为每项添加点击事件的命令
        // 注：此处的属性赋值在 constructor 阶段 super 语句之后执行
        this.command = {
            title: 'Open ' + this.label,
            command: 'configExplorer.openConfig',
            arguments: [
                // 向 registerCommand 传递的参数。
                {
                    path: this.path,
                    parent: this.parent,
                    detail: utils.deepClone(this.detail)
                }
            ]
        };
        this.iconPath = {
            light: utils.lightRes('file.svg'),
            dark: utils.darkRes('file.svg')
        };
        this.contextValue = this.genContext();
    }
    get path() {
        return this.parent.path;
    }
    get configName() {
        return utils.getConfigName(this.detail);
    }
    get fullPath() {
        return libPath.join(utils.getRoot(vscode), this.path, this.configName);
    }
    get sha256() {
        let hash = utils.getProp(this, 'detail.hash_value') || '';
        return hash;
    }
    genContext() {
        if (this.collapsibleState !== vscode.TreeItemCollapsibleState.None) {
            return 'dirItem';
        }
        if (this.isExist) {
            if (this.detail.tag) {
                return 'fileItem_tag';
            }
            else {
                return 'fileItem_notag';
            }
        }
        return '';
    }
}
exports.TreeItem = TreeItem;
//# sourceMappingURL=config-item.js.map