"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigTreeProvider = void 0;
const vscode = __importStar(require("vscode"));
const libPath = __importStar(require("path"));
const utils = __importStar(require("./utils"));
const consts = __importStar(require("./constant"));
const config_item_1 = require("./config-item");
class ConfigTreeProvider {
    constructor(rootConfig, spaceState) {
        this.rootConfig = rootConfig;
        this.spaceState = spaceState;
        this._onDidChangeTreeData = new vscode.EventEmitter();
        this.onDidChangeTreeData = this._onDidChangeTreeData.event;
        this.initTree();
    }
    getTreeItem(element) {
        return element;
    }
    // 关键，获取子节点
    getChildren(element) {
        // 当 element 不存在时，表示是根节点，把总配置列表传入
        if (!element) {
            let configs = this.rootConfig.map((config) => {
                return new config_item_1.TreeEnv(config.environment_name, vscode.TreeItemCollapsibleState.Collapsed, config, this.spaceState);
            });
            return Promise.resolve(configs);
        }
        else {
            if (element.detail) {
                if (element.detail.hasOwnProperty(consts.configKey)) {
                    // 父级为 Env 节点，当前为 App 节点
                    let folders = element.detail[consts.configKey].map((config) => {
                        return new config_item_1.TreeApp(config.application_name, vscode.TreeItemCollapsibleState.Collapsed, element, {
                            server: element.detail.domain,
                            authkey: element.detail.operator,
                            appid: config.application_id,
                            collections: [],
                            deployment: config.collections || [],
                            ignore_collections: config.ignore_collections || []
                        });
                    });
                    return Promise.resolve(folders);
                }
                else if (element.detail.hasOwnProperty('collections')) {
                    // 父级为 App 节点，当前为 Collect 节点
                    return new Promise((resolve, reject) => {
                        utils
                            .getCollectList(element.detail)
                            .then((result) => {
                            let folders = result
                                .filter((config) => {
                                let ignore = utils.getProp(element, 'detail.ignore_collections') || [];
                                return ignore.indexOf(config) === -1;
                            })
                                .sort()
                                .map((config) => {
                                let publishConfig = utils.getProp(element, `detail.deployment.${config}`);
                                return new config_item_1.TreeCollect(config, vscode.TreeItemCollapsibleState.Collapsed, element, {
                                    server: element.detail.server,
                                    authkey: element.detail.authkey,
                                    app: element.label,
                                    appid: element.detail.appid,
                                    collect: config,
                                    publish: publishConfig
                                });
                            });
                            resolve(folders);
                        })
                            .catch((err) => {
                            reject(err);
                            utils.createLogFile({
                                type: 'getCollectList',
                                message: err
                            });
                        });
                    });
                }
                else if (element.detail.hasOwnProperty('children')) {
                    // 父级为 Tag 节点，当前为 Item 节点
                    let items = this.genTreeItemList(element.detail['children'], element);
                    return Promise.resolve(items);
                }
                else {
                    // 父级为 Collect 节点，当前为 Tag 或 Item 节点
                    return new Promise((resolve, reject) => {
                        utils
                            .getConfigList(element.detail)
                            .then((result) => {
                            let child = this.genTreeCollectChildren(result, element);
                            resolve(child);
                        })
                            .catch((err) => {
                            reject(err);
                            utils.createLogFile({
                                type: 'getConfigList',
                                message: err
                            });
                        });
                    });
                }
            }
            return Promise.resolve([]);
        }
    }
    genTreeCollectChildren(result, element) {
        let tagList = [];
        let agent = {
            [consts.untagName]: []
        };
        result.forEach((item) => {
            if (item.tag) {
                if (tagList.indexOf(item.tag) === -1) {
                    tagList.push(item.tag);
                    agent[item.tag] = [];
                }
                agent[item.tag].push(item);
            }
            else {
                agent[consts.untagName].push(item);
            }
        });
        tagList.sort();
        let tags = tagList.map((tag) => {
            return new config_item_1.TreeTag(tag, vscode.TreeItemCollapsibleState.Collapsed, element, {
                children: agent[tag]
            });
        });
        let items = this.genTreeItemList(agent[consts.untagName], element);
        return tags.concat(items);
    }
    genTreeItemList(result, element) {
        return result
            .sort((a, b) => a.config_id - b.config_id)
            .map((config) => {
            let name = utils.getConfigName(config);
            let label = config.config_id + '(' + name + ')';
            return new config_item_1.TreeItem(label, vscode.TreeItemCollapsibleState.None, element, config);
        });
    }
    initTree(type) {
        if (type) {
            utils.msg(consts.msgText.checking);
        }
        let limitTaskHandle = utils.genLimitTask(this, 20);
        let downloadInterval = {};
        this.rootConfig.forEach((env) => __awaiter(this, void 0, void 0, function* () {
            let envName = env.environment_name;
            if (!env.operator) {
                utils.msg(consts.msgText.noOperator(envName), consts.msgType.warn);
                console.log(148);
            }
            utils.createFolder(envName);
            let spaceState = this.spaceState;
            env[consts.configKey].forEach((config) => {
                let detail = {
                    env: envName,
                    server: env.domain,
                    authkey: env.operator,
                    app: config.application_name,
                    appid: config.application_id,
                    collect: '',
                    publish: ''
                };
                utils
                    .getCollectList(detail)
                    .then((collections) => {
                    collections
                        .filter((collect) => {
                        let ignore = utils.getProp(config, 'ignore_collections') || [];
                        return ignore.indexOf(collect) === -1;
                    })
                        .forEach((collect) => {
                        let clone = utils.deepClone(detail);
                        clone.collect = collect;
                        clone.publish = utils.getProp(config, `collections.${collect}`);
                        let filePath = libPath.join(envName, config.application_name, collect);
                        let configPath = `_${envName}_${config.application_name}_${collect}`;
                        let label = filePath + '：';
                        utils.createFolder(filePath);
                        // init inner config file of collect
                        spaceState.update('config_collect' + configPath, JSON.stringify(clone));
                        spaceState.update('config_collect_update' + configPath, Date.now() - 0);
                        console.log(182, clone.authkey);
                        // init inner config file of commit code
                        let configItemPath = 'config_item' + configPath;
                        let configItemObj = utils.readConfigState(spaceState, configItemPath);
                        configItemObj._realId = configItemPath;
                        // disable download json file when init extension
                        if (!type) {
                            return;
                        }
                        utils
                            .getConfigList(clone)
                            .then((result) => __awaiter(this, void 0, void 0, function* () {
                            let downloadList = result;
                            // 非强制下载模式时，需要获取所有需要更新的条目，用于后期判断是否全部处理完毕
                            if (type !== 'force') {
                                downloadList = yield utils.getDownloadList(result, {
                                    detail: clone,
                                    configItemObj,
                                    path: filePath
                                });
                            }
                            if (type === 'update' && !downloadList.length) {
                                utils.msg(label + consts.msgText.downloadStop, consts.msgType.warn);
                                return;
                            }
                            let flag = {
                                count: 0,
                                pass: [],
                                stop: []
                            };
                            if (type) {
                                utils.msg(label + consts.msgText.downloadAll(downloadList));
                            }
                            if (downloadList.length > 10) {
                                downloadInterval[label] = setInterval(() => {
                                    utils.msg(label + ' 已下载 ' + flag.pass.length + '/' + downloadList.length);
                                }, 5000);
                            }
                            downloadList.forEach((item) => {
                                let name = utils.getConfigName(item);
                                let fileDetail = Object.assign(Object.assign(Object.assign({}, clone), item), { force: type });
                                limitTaskHandle({
                                    fileDetail,
                                    filePath,
                                    name,
                                    label,
                                    type,
                                    configItemObj,
                                    flag,
                                    length: downloadList.length
                                }, this.callDownload, (target, configItemObj) => {
                                    configItemObj[target.config_id] = target.commit_code;
                                }, (label, configItemObj) => {
                                    spaceState.update(configItemObj._realId, JSON.stringify(configItemObj));
                                    clearInterval(downloadInterval[label]);
                                    this.refresh();
                                });
                            });
                        }))
                            .catch((err) => {
                            utils.createLogFile({
                                type: 'initConfigList',
                                message: err
                            });
                        });
                    });
                })
                    .catch((err) => {
                    utils.createLogFile({
                        type: 'initCollectList',
                        message: err
                    });
                });
            });
        }));
    }
    clearState() {
        this.rootConfig.forEach((env) => __awaiter(this, void 0, void 0, function* () {
            let envName = env.environment_name;
            let spaceState = this.spaceState;
            env[consts.configKey].forEach((config) => {
                let detail = {
                    env: envName,
                    server: env.domain,
                    authkey: env.operator,
                    app: config.application_name,
                    appid: config.application_id,
                    collect: '',
                    publish: ''
                };
                utils
                    .getCollectList(detail)
                    .then((collections) => {
                    collections
                        .filter((collect) => {
                        let ignore = utils.getProp(config, 'ignore_collections') || [];
                        return ignore.indexOf(collect) === -1;
                    })
                        .forEach((collect) => {
                        let configPath = `_${envName}_${config.application_name}_${collect}`;
                        // reset inner config file of commit code
                        let configItemPath = 'config_item' + configPath;
                        spaceState.update(configItemPath, undefined);
                    });
                })
                    .catch((err) => { });
            });
        }));
        this.refresh();
        utils.msg(consts.msgText.resetState);
    }
    callDownload({ fileDetail, filePath, name, label, type, configItemObj, flag, length }, callback, lastCallback) {
        return utils
            .createConfigFile(fileDetail, filePath, name)
            .then(() => {
            flag.pass.push(fileDetail.config_id);
            callback && callback(fileDetail, configItemObj);
        })
            .catch(() => {
            flag.stop.push(fileDetail.config_id);
        })
            .finally(() => {
            flag.count++;
            if (flag.count >= length && type) {
                if (flag.pass.length === length) {
                    let msg = label + consts.msgText.downloadAllOk({ length });
                    utils.msg(msg);
                }
                else {
                    let msg = label + consts.msgText.downloadFail + '：' + flag.stop.join('/');
                    utils.msg(msg, consts.msgType.error);
                }
                lastCallback && lastCallback(label, configItemObj);
            }
        });
    }
    open(element) {
        vscode.window.showTextDocument(vscode.Uri.file(element.path));
    }
    refresh() {
        this._onDidChangeTreeData.fire(undefined);
    }
}
exports.ConfigTreeProvider = ConfigTreeProvider;
//# sourceMappingURL=config-provider.js.map