"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode = __importStar(require("vscode"));
const libPath = __importStar(require("path"));
const utils = __importStar(require("./utils"));
const consts = __importStar(require("./constant"));
const file_provider_1 = __importDefault(require("./file-provider"));
const config_explorer_1 = __importDefault(require("./config-explorer"));
const generator_1 = __importDefault(require("./generator"));
const json_path_1 = require("./json-path");
let currentString = '';
let statusbar;
function activate(context) {
    let agent = {};
    let detailPage = vscode.commands.registerCommand('extension.showDetail', () => {
        agent.panel = utils.createWebview(context, 'Detail', vscode.ViewColumn.Active, 'html/config-detail.html', (msg) => {
            if (utils.messageHandler[msg.cmd]) {
                utils.messageHandler[msg.cmd](agent, msg);
            }
        });
    });
    context.subscriptions.push(detailPage);
    // 定义配置列表展示的 webview 页
    let configListPage = vscode.commands.registerCommand('extension.showLink', () => {
        agent.panel = utils.createWebview(context, 'Links', vscode.ViewColumn.Active, 'html/config-detail.html', (msg) => {
            if (utils.messageHandler[msg.cmd]) {
                utils.messageHandler[msg.cmd](agent, msg);
            }
        });
    });
    context.subscriptions.push(configListPage);
    // 读取项目目录下插件设置文件
    let rootPath = utils.getRoot(vscode);
    if (rootPath) {
        // 初始化本地文件树
        new file_provider_1.default(rootPath, context);
        // 读取基础设置文件，初始化远程配置树
        let configPath = libPath.join(rootPath, consts.rootConfigFolder, consts.baseConfigName);
        utils.readFile(configPath, (result) => {
            let json = JSON.parse(result);
            new config_explorer_1.default(json.environments, context, agent);
        });
        // 读取转换设置文件，成功后提供批量生成配置文件功能
        let genConfigPath = libPath.join(rootPath, consts.rootConfigFolder, consts.transConfigName);
        utils.readFile(genConfigPath, (result) => {
            vscode.commands.executeCommand('setContext', 'showGenerator', true);
            new generator_1.default(genConfigPath);
        }, (err) => {
            vscode.commands.executeCommand('setContext', 'showGenerator', false);
        });
    }
    else {
        utils.msg(consts.msgText.noWorkspace, consts.msgType.warn);
    }
    // JSON path module
    statusbar = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 100);
    statusbar.command = 'extension.copyJSONPath';
    statusbar.show();
    context.subscriptions.push(statusbar, vscode.window.onDidChangeActiveTextEditor(() => updateStatus(statusbar)), vscode.window.onDidChangeTextEditorSelection(() => updateStatus(statusbar)), vscode.commands.registerCommand('extension.copyJSONPath', () => __awaiter(this, void 0, void 0, function* () {
        yield vscode.env.clipboard.writeText(currentString);
    })));
    updateStatus(statusbar);
}
exports.activate = activate;
function updateStatus(status) {
    currentString = '';
    const editor = vscode.window.activeTextEditor;
    if (!editor || !(editor.document.languageId.toLowerCase() === 'json' || editor.document.languageId.toLowerCase() === 'jsonc')) {
        status.text = '';
        return;
    }
    try {
        const text = editor.document.getText();
        const path = (0, json_path_1.jsonPathTo)(text, editor.document.offsetAt(editor.selection.active));
        currentString = path;
        status.text = 'Path: ' + path;
        status.tooltip = 'Click to copy to clipboard';
    }
    catch (err) {
        if (err instanceof SyntaxError) {
            status.text = 'Path: Invalid JSON.';
        }
        else {
            status.text = 'Path: Error.';
        }
        status.tooltip = undefined;
    }
}
function deactivate() { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map