"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = __importStar(require("vscode"));
const libPath = __importStar(require("path"));
const libFs = __importStar(require("fs"));
const utils = __importStar(require("./utils"));
const consts = __importStar(require("./constant"));
const file_item_1 = __importDefault(require("./file-item"));
class FileTreeProvider {
    // 从外部传入根路径
    constructor(rootDir) {
        this.rootDir = rootDir;
        this._onDidChangeTreeData = new vscode.EventEmitter();
        this.onDidChangeTreeData = this._onDidChangeTreeData.event;
    }
    getTreeItem(element) {
        return element;
    }
    // 关键，获取子节点
    getChildren(element) {
        // 当 element 不存在，则代表它是根节点，把根路径 this.rootDir 放进去，并默认展开
        if (!element) {
            return Promise.resolve([new file_item_1.default(this.rootDir, vscode.TreeItemCollapsibleState.Expanded, null)]);
        }
        else {
            let items = [];
            // 否则遍历文件夹
            let subfiles = libFs.readdirSync(element.path);
            subfiles.forEach((file) => {
                let stat = libFs.statSync(libPath.join(element.path, file));
                // 如果是目录则设置它可以展开，否则设置它不可展开
                items.push(new file_item_1.default(file, stat.isDirectory() ? vscode.TreeItemCollapsibleState.Collapsed : vscode.TreeItemCollapsibleState.None, element));
            });
            return Promise.resolve(items);
        }
    }
    open(element) {
        vscode.window.showTextDocument(vscode.Uri.file(element.path));
    }
    delete(element) {
        let path = element.path;
        let stat = libFs.statSync(path);
        if (stat.isDirectory()) {
            utils.deleteFolder(path);
        }
        else {
            libFs.unlinkSync(path);
        }
        this.refresh();
    }
    refresh() {
        this._onDidChangeTreeData.fire(undefined);
    }
    upload(element, props) {
        let post = {
            server: props.server,
            authkey: props.authkey,
            appid: props.appid,
            collect: props.collect,
            filepath: element.path,
            param: {
                config_id: props.id,
                description: props.desc,
                hash_value: props.hash,
                tag: props.tag
            }
        };
        console.log(70, props.authkey);
        return;
        utils
            .uploadConfig(post)
            .then((result) => {
            props.configItemObj[result.config_id] = result.commit_code;
            props.spaceState.update(props.configItemPath, JSON.stringify(props.configItemObj));
            let publishUrl = props.publish;
            return utils.prePublish(publishUrl, result, post);
        })
            .then(() => {
            utils.msg(consts.msgText.fileAddOk);
        })
            .catch((err) => {
            let errMsg = consts.msgText.fileUploadFail;
            if (err.interface) {
                errMsg = err.msg.msg;
            }
            utils.msg(errMsg, consts.msgType.error);
            utils.createLogFile({
                type: 'newConfig',
                message: err
            });
        });
    }
}
class FileExplorer {
    // private treeViewer: vscode.TreeView<NodeItem>;
    constructor(root, context) {
        this.context = context;
        this.provider = new FileTreeProvider(root);
        // this.treeViewer = vscode.window.createTreeView('hiFileTree', {treeDataProvider: this.provider});
        context.subscriptions.push(vscode.commands.registerCommand('fileExplorer.open', (node) => {
            this.provider.open(node);
        }), vscode.commands.registerCommand('fileExplorer.new', (node) => {
            vscode.commands.executeCommand('explorer.newFile', node);
        }), vscode.commands.registerCommand('fileExplorer.delete', (node) => {
            this.provider.delete(node);
        }), vscode.commands.registerCommand('fileExplorer.refresh', (node) => {
            this.provider.refresh();
        }), vscode.commands.registerCommand('fileExplorer.upload', (node) => {
            this.upload(node);
        }));
    }
    upload(node) {
        let configId = '';
        let tagName = '';
        let options = {
            prompt: 'config_id:',
            placeHolder: consts.hintText.configId
        };
        let spaceState = this.context.workspaceState;
        let configPath = utils.genCommonConfigPath(libPath.dirname(node.path));
        // check valid
        let buffer = libFs.readFileSync(node.path);
        let str = buffer.toString();
        try {
            JSON.parse(str);
        }
        catch (err) {
            utils.msg(consts.msgText.notJson, consts.msgType.warn);
            return;
        }
        let json = utils.readConfigState(spaceState, 'config_collect' + configPath);
        if (!utils.isEmpty(json)) {
            let time = utils.readConfigState(spaceState, 'config_collect_update' + configPath);
            if (time) {
                console.log(150, json.authkey);
                let duration = (Date.now() - time) / 1000;
                console.log(153, duration);
            }
            let configItemPath = 'config_item' + configPath;
            let configItemObj = utils.readConfigState(spaceState, configItemPath);
            vscode.window
                .showInputBox(options)
                .then((value) => {
                if (utils.checkValue(value, true)) {
                    return Promise.reject();
                }
                if (!utils.isNumber(value)) {
                    utils.msg(consts.msgText.notNumber, consts.msgType.warn);
                    return Promise.reject();
                }
                configId = value || '';
                let arr = Object.keys(configItemObj);
                if (arr.indexOf(configId) > -1) {
                    utils.msg(consts.msgText.notUnique, consts.msgType.warn);
                    return Promise.reject();
                }
                let options = {
                    prompt: 'tag_name:',
                    placeHolder: consts.hintText.tag
                };
                return vscode.window.showInputBox(options);
            })
                .then((value) => {
                if (utils.checkValue(value)) {
                    return Promise.reject();
                }
                tagName = (value === null || value === void 0 ? void 0 : value.toLowerCase()) || '';
                let options = {
                    prompt: 'description:',
                    placeHolder: consts.hintText.desc
                };
                return vscode.window.showInputBox(options);
            })
                .then((value) => {
                if (utils.checkValue(value)) {
                    return;
                }
                this.provider.upload(node, Object.assign({ id: configId, tag: tagName, desc: value || '', spaceState,
                    configItemObj,
                    configItemPath }, json));
            })
                .then(undefined, (err) => { });
        }
        else {
            utils.msg(consts.msgText.noSetting, consts.msgType.warn);
        }
    }
}
exports.default = FileExplorer;
//# sourceMappingURL=file-provider.js.map