"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = __importStar(require("vscode"));
const libPath = __importStar(require("path"));
const libFs = __importStar(require("fs"));
const utils = __importStar(require("./utils"));
const consts = __importStar(require("./constant"));
class ConfigGenerator {
    constructor(rootConfig) {
        vscode.commands.registerCommand('configExplorer.generateConfig', (node) => {
            utils.readFile(rootConfig, (result) => {
                vscode.commands.executeCommand('setContext', 'showGenerator', true);
                let json = JSON.parse(result);
                this.generate(json, node);
            });
        });
    }
    generate(configs, folder) {
        if (!Array.isArray(configs)) {
            return;
        }
        let rootPath = utils.getRoot(vscode);
        // 指定文件夹时记录该目录相关的有效配置数
        let total = configs.length;
        // 描述有效的配置数
        let count = 0;
        let invalid = false;
        // parse generator config
        configs.forEach((item, index) => {
            let { base_config, target_config, filter_config, factor } = item;
            if (!base_config || !target_config || !filter_config || !factor) {
                utils.msg(`设置文件第 ${index + 1} 项缺少必要属性`, consts.msgType.warn);
                return;
            }
            let sourcePath = libPath.join(rootPath, base_config);
            if (folder && sourcePath.indexOf(folder.path + '/') === -1) {
                total--;
                return;
            }
            let baseConfig = {};
            let filters = [];
            utils.readFile(sourcePath, (result) => {
                let json = JSON.parse(result);
                baseConfig = json;
            }, () => {
                utils.msg(`设置文件第 ${index + 1} 项 base_config 路径有误`, consts.msgType.warn);
                invalid = true;
            });
            utils.readFile(libPath.join(rootPath, filter_config), (result) => {
                let arr = result.split('\n');
                filters = arr.filter((rule) => rule.trim());
                if (!filters.length) {
                    utils.msg(`设置文件第 ${index + 1} 项描述文件内容为空，未生成新的配置文件。`, consts.msgType.warn);
                    invalid = true;
                }
            }, () => {
                utils.msg(`设置文件第 ${index + 1} 项 filter_config 路径有误`, consts.msgType.warn);
                invalid = true;
            });
            if (invalid) {
                invalid = false;
                return;
            }
            let json = this.transformConfig(baseConfig, filters, factor, index + 1);
            if (json) {
                count++;
                let targetPath = libPath.join(rootPath, target_config);
                this.writeConfigFile(json, targetPath);
            }
        });
        if (count) {
            // 假如任一配置有效并执行就会通知
            utils.msg(consts.msgText.transformOk);
        }
        else if (folder && !total) {
            // 假如指定文件夹，且没有任何配置包含该文件夹就会通知
            utils.msg(consts.msgText.noIncludeFolder, consts.msgType.warn);
        }
    }
    transformConfig(config, filters, factor, index) {
        let clone = JSON.parse(JSON.stringify(config));
        if (typeof factor !== 'number') {
            return;
        }
        let exception = [];
        filters.forEach((path) => {
            let value = utils.getProp(clone, utils.arr2dot(path));
            if (utils.isNil(value)) {
                exception.push(path);
                return;
            }
            if (typeof value === 'number') {
                utils.setProp(clone, utils.arr2dot(path), this.getInteger(value, factor));
            }
            else if (Array.isArray(value)) {
                let arr = value.map((item) => {
                    return this.getInteger(item, factor);
                });
                utils.setProp(clone, utils.arr2dot(path), arr);
            }
            else if (typeof value === 'object') {
                Object.keys(value).map((key) => {
                    let now = value[key];
                    value[key] = this.getInteger(now, factor);
                });
            }
        });
        if (exception.length) {
            utils.msg(`设置文件第 ${index} 项描述文件内容有路径错误，未生成新的配置文件。`, consts.msgType.warn);
            return null;
        }
        return clone;
    }
    getInteger(num, factor) {
        let result = num * factor;
        if (!result) {
            return num;
        }
        if (result < 1) {
            result = 1;
        }
        else {
            result = Math.floor(result);
        }
        // 取消按两位取整处理功能
        /*
        let str = result.toFixed(0);
        let hundred = Math.pow(10, str.length - 2);
        result = Math.floor(Number(str) / hundred) * hundred;
        */
        return result;
    }
    writeConfigFile(content, fullPath) {
        if (!libFs.existsSync(fullPath)) {
            libFs.mkdirSync(libPath.dirname(fullPath), { recursive: true });
        }
        libFs.writeFile(fullPath, JSON.stringify(content, null, 4), (err) => {
            if (err) {
                utils.msg(err.message, consts.msgType.error);
            }
        });
    }
}
exports.default = ConfigGenerator;
//# sourceMappingURL=generator.js.map